package enquete.model;

/*
 * Historico:
 *    24/03/2003: rodrigor - Criao da classe
 *    26/03/2003: rodrigor - Alterado o mtodo disparaNovoVoto() para receber uma opo
 *                           Alterado o mtodo disparaNovaOpcao() para receber uma opo
 *    23/03/2006: renata - Alterado cdico para adicionar Generics do Java 5.0
 */


import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Armazena opes a serem votadas,
 * incrementa votos e retorna os totais da votao. Esta classe
 * representa o modelo de negcio no exemplo sobre Model-View-Controller
 *
 * @author <a href="mailto:rodrigor@dsc.ufcg.edu.br">Rodrigo Rebouas de Almeida</a>
 * @since 24/03/2003
 **/
public class EnqueteSimples {

	private Map <String, Integer> opcoes;
	private List <EnqueteListener>enqueteListeners = new LinkedList();

	public EnqueteSimples(){
		opcoes = new HashMap<String, Integer>();
	}

	/**
	 * Adiciona uma opo para ser votada na enquete
	 * @param opcao nome da opo
	 */
	public void addOpcao(String opcao){
		opcoes.put(opcao,new Integer(0));
		this.disparaNovaOpcao(opcao);
	}

	/**
	 * Retorna um iterador de opes disponveis na enquete
	 * @return Iterator opes disponiveis na enquete
	 */
	public Set <String> getOpcoes(){
		return opcoes.keySet();
	}

	/**
	 * Incrementa um voto para opo
	 * @param opcao opo que receber voto
	 */
	public void votar(String opcao){
		int votoAtual = (opcoes.get(opcao)).intValue();
		opcoes.put(opcao,new Integer(++votoAtual));
		this.disparaNovoVoto(opcao);
	}

	/**
	 * Retorna a soma dos votos de todas as opes da enquete
	 * @return int soma dos votos de todas as opes da enquete
	 */
	public int getTotalVotos(){
		int total = 0;
		for(Integer votos : opcoes.values()){
			total+= votos.intValue();
		}
		return total;
	}

	/**
	 * Retorna a quantidade de votos de uma opo individual
	 * @param opcao opo que se quer o voto
	 * @return int quantidade de votos da opo
	 */
	public int getVotos(String opcao){
		return (opcoes.get(opcao)).intValue();
	}


	/**
	 * Adiciona um EnqueteListener, um objeto interessado em
	 * receber eventos lanados pela Enquete
	 * @see EnqueteListener
	 * @param listener objeto interessado em receber eventos
	 */
	public synchronized void addEnqueteListener(EnqueteListener listener){
		if(enqueteListeners.contains(listener)){ return; }
		this.enqueteListeners.add(listener);
	}

	/**
	 * Informa aos objetos interessados nos eventos lanados
	 * pela Enquete que um novo voto foi contabilizado.
	 */
	private synchronized void disparaNovoVoto(String opcao){		
		for(EnqueteListener listeners : this.enqueteListeners){
			listeners.novoVoto(new EnqueteEvent(this,opcao));
		}
	}

	/**
	 * Informa aos objetos interessados nos enventos lanados
	 * pela Enquete que uma nova opo foi adicionada.
	 */
	private synchronized void disparaNovaOpcao(String opcao){		
		for(EnqueteListener listeners : this.enqueteListeners){
			listeners.novaOpcao(new EnqueteEvent(this,opcao));
		}
	}

}
